#!/usr/bin/env python

import sys
import os
from twython import Twython
import io
import qrcode
import time
import serial
import signal

ENABLE_TWITTER = False
ENABLE_LUMA = True
ENABLE_PEOPLE_COUNTER = True
START_BUTTON_PIN = 7
SAVE_IMAGES = True


def signal_handler(signal, frame):

    print("You pressed Ctrl+C!")
        
    # unexport pin 
    f = open('/sys/class/gpio/unexport', 'w')
    f.write(str(START_BUTTON_PIN))
    f.close()

    print("Goodbye!")
    
    sys.exit(0)

# set up sigint handler
signal.signal(signal.SIGINT, signal_handler)
#signal.pause()


# set up luma lighting
if ENABLE_LUMA:
    try:
        ser = serial.Serial(port = '/dev/ttyUSB0', baudrate=19200)
        ser.write("[01002]".encode())
        ser.write("[01200FF00]".encode())
        time.sleep(0.50)
        ser.write("[012444444]".encode())
    except Exception as e:
        ENABLE_LUMA = False

# hacky code to generate Twitter authentication keys
# to use, start with app key and complete step 1, then use result for step 2, etc.

# Twitter API app key and secret key
APP_KEY = ''
APP_SECRET = ''

# step 1
if 0 == 1:
    twitter = Twython(APP_KEY, APP_SECRET)
    auth = twitter.get_authentication_tokens()
    print("Go to this url and get the oauth_verifier. Also use the token and secret printed below and update this file.")
    print(auth['auth_url'])

    OAUTH_TOKEN = auth['oauth_token']
    OAUTH_TOKEN_SECRET = auth['oauth_token_secret']
    print(OAUTH_TOKEN, OAUTH_TOKEN_SECRET)

    assert 1 == 0

# step 2
if 0 == 1:

    OAUTH_TOKEN = ''
    OAUTH_TOKEN_SECRET = ''

    oauth_verifier = 't5cI8WpGHYpgKC3EB8rIjL5OvU3wLMlxYlEQwjUbc'

    twitter = Twython(APP_KEY, APP_SECRET, OAUTH_TOKEN, OAUTH_TOKEN_SECRET)
    final_step = twitter.get_authorized_tokens(oauth_verifier)

    OAUTH_TOKEN = final_step['oauth_token']
    OAUTH_TOKEN_SECRET = final_step['oauth_token_secret']

    print("Final tokens ", OAUTH_TOKEN, OAUTH_TOKEN_SECRET)
    print("Put these tokens in the python file!")

    assert 1 == 0

# authenticate Twitter
if ENABLE_TWITTER:

    OAUTH_TOKEN = ''
    OAUTH_TOKEN_SECRET = ''

    twitter = Twython(APP_KEY, APP_SECRET, OAUTH_TOKEN, OAUTH_TOKEN_SECRET)

# keep track of the number of uses
if ENABLE_PEOPLE_COUNTER:
    counter = 0;
    try:
        uses = open('/root/uses', 'r')
        counter = int(uses.readline())
        uses.close()
    except Exception as e:
        print(e)

# set up gpio
try:
    # unexport pin just for fun
    os.system("echo " + str(START_BUTTON_PIN) + " > /sys/class/gpio/unexport")
except Exception as e:
    print("Error unexporting")
    pass

# export pin number
f = open('/sys/class/gpio/export', 'w')
f.write(str(START_BUTTON_PIN))
f.close()

# define pin direction as input
button_path = '/sys/class/gpio/gpio' + str(START_BUTTON_PIN) + '/direction'
f = open(button_path, 'w')
f.write('in')
f.close()

button_path = '/sys/class/gpio/gpio' + str(START_BUTTON_PIN) + '/value'

print("Starting photobooth!")

while (1):

	# set mood lighting
    if ENABLE_LUMA:
        ser.write("[01002]".encode())
        ser.write("[012444444]".encode())

	# read in button pin
    f = open(button_path, 'r')
    button_position = f.read()
    #print(button_position)
    
    if("1" in button_position):
	
        print("Button pressed!")
    
        try:
	        # take pictures
            print("Taking pictures...")
            if ENABLE_LUMA:
                ser.write("[012FFFFFF]".encode())
            os.system("cd /tmp; gphoto2 --capture-image-and-download --keep --interval=3 --frames=4 --filename /tmp/portrait%n.jpg --force-overwrite")
            if ENABLE_LUMA:
                ser.write("[012444444]".encode())
                ser.write("[012FF6A00]".encode())
	    	
            # assemble images
            print("Converting images...")
            cmd = "convert /tmp/portrait1.jpg -sample 668 -bordercolor '#FFFFFF' -border 2x20 /tmp/portrait1.jpg & "
            
            #print("Convert 2")
            cmd += " convert /tmp/portrait2.jpg -sample 668 -bordercolor '#FFFFFF' -border 2x20 /tmp/portrait2.jpg & "
            cmd += "wait"
            os.system(cmd)
	
            #print("Convert 3")
            cmd = " convert /tmp/portrait3.jpg -sample 668 -bordercolor '#FFFFFF' -border 2x20 /tmp/portrait3.jpg & "
            
            #print("Convert 4")
            cmd += " convert /tmp/portrait4.jpg -sample 668 -bordercolor '#FFFFFF' -border 2x20 /tmp/portrait4.jpg & "
            cmd += "wait"
            
            os.system(cmd)

            #print("Appending")
            os.system("convert -append /tmp/portrait1.jpg /tmp/portrait2.jpg /tmp/portrait3.jpg /tmp/portrait4.jpg /tmp/twitter.jpg")
            
            
	        # send to twitter
            if ENABLE_TWITTER:
			
                print("Sending to Twitter...")
				
                if ENABLE_LUMA:
                    ser.write("[007FF0000]".encode())
                
                try:
                    photo = open("/tmp/twitter.jpg", 'rb')
                    header = twitter.update_status_with_media(media=photo, status='#protofusion photobooth at #makerfairedetroit')
                    
                    if ENABLE_LUMA:
                        ser.write("[0120000FF]".encode())

	    	        # generate qr code
                    print("Generating QR code...")
                    img = qrcode.make(header['extended_entities']['media'][0]['url'])
                    img.save("/tmp/qrcode.jpg", "JPEG")
                    os.system("convert /tmp/qrcode.jpg -sample 245 /tmp/qrcode.jpg")
                    os.system("convert +append /tmp/qrcode.jpg /root/photoboothtext.jpg /tmp/qrblock.jpg")
	    	
                except Exception as e:
                    print(e)
					
                    if ENABLE_LUMA:
                        ser.write("[007FF0000]".encode())
						
                    os.system("convert /root/protocode.jpg -sample 245 /tmp/qrcode.jpg")
                    os.system("convert +append /tmp/qrcode.jpg /root/photoboothtextnotwitter.jpg /tmp/qrblock.jpg")
	    	
            else:
                os.system("convert /root/protocode.jpg -sample 245 /tmp/qrcode.jpg")
                os.system("convert +append /tmp/qrcode.jpg /root/photoboothtextnotwitter.jpg /tmp/qrblock.jpg")
                pass
                
                
	    	# assemble images and print
            print("Printing photo...")
            os.system("convert -append /tmp/twitter.jpg /root/protofusion.jpg /tmp/print.jpg")
            os.system("lpr -o orientation-requested=3 /tmp/print.jpg")
            if SAVE_IMAGES:
                os.system("cp /tmp/twitter.jpg /root/images/photostrip" + str(counter) + ".jpg")
	    	
            # delete everything once we're done
            print("Cleaning up...")
            os.system("rm /tmp/portrait1.jpg")
            os.system("rm /tmp/portrait2.jpg")
            os.system("rm /tmp/portrait3.jpg")
            os.system("rm /tmp/portrait4.jpg")
            #os.system("rm /tmp/qrcode.jpg")
            os.system("rm /tmp/twitter.jpg")
            #os.system("rm /tmp/print.jpg")
            #os.system("rm /tmp/qrblock.jpg")
            
            if ENABLE_LUMA:
                ser.write("[012444444]".encode())
            
            counter += 1
            
            # write back the number of uses
            if ENABLE_PEOPLE_COUNTER:
                try:
                    uses = open("/root/uses", "w")
                    uses.write(str(counter))
                    uses.close()
                except Exception as e:
                    print(e)
                
            print("Done!")
    
        except Exception as e:
            print(e)
    	
    f.close()
    	
    time.sleep(0.10)